@@ -9,6 +9,8 @@ import os
 import re
 import sys
 import traceback
+import ast
+import importlib
 from re import sub, findall
 from types import CodeType
 from functools import partial
@@ -35,11 +37,16 @@ Cache.register('kv.lang')
 __KV_INCLUDES__ = []
 
 # precompile regexp expression
-lang_str = re.compile(
-    "((?:'''.*?''')|"
+str_re = (
+    "(?:'''.*?''')|"
     "(?:(?:(?<!')|''')'(?:[^']|\\\\')+?'(?:(?!')|'''))|"
     '(?:""".*?""")|'
-    '(?:(?:(?<!")|""")"(?:[^"]|\\\\")+?"(?:(?!")|""")))', re.DOTALL)
+    '(?:(?:(?<!")|""")"(?:[^"]|\\\\")+?"(?:(?!")|"""))'
+)
+
+lang_str = re.compile(f"({str_re})", re.DOTALL)
+lang_fstr = re.compile(f"([fF](?:{str_re}))", re.DOTALL)
+
 lang_key = re.compile('([a-zA-Z_]+)')
 lang_keyvalue = re.compile(r'([a-zA-Z_][a-zA-Z0-9_.]*\.[a-zA-Z0-9_.]+)')
 lang_tr = re.compile(r'(_\()')
@@ -191,14 +198,21 @@ class ParserRuleProperty(object):
             return
 
         # now, detect obj.prop
+        # find all the fstrings in the  value
+        fstrings = lang_fstr.findall(value)
+        wk = set()
+        for s in fstrings:
+            expression = ast.parse(s)
+            wk |= set(self.get_names_from_expression(expression.body[0].value))
+
         # first, remove all the string from the value
         tmp = sub(lang_str, '', value)
         idx = tmp.find('#')
         if idx != -1:
             tmp = tmp[:idx]
         # detect key.value inside value, and split them
-        wk = list(set(findall(lang_keyvalue, tmp)))
-        if len(wk):
+        wk |= set(findall(lang_keyvalue, tmp))
+        if wk:
             self.watched_keys = [x.split('.') for x in wk]
         if findall(lang_tr, tmp):
             if self.watched_keys:
@@ -206,6 +220,73 @@ class ParserRuleProperty(object):
             else:
                 self.watched_keys = [['_']]
 
+    @classmethod
+    def get_names_from_expression(cls, node):
+        """
+        Look for all the symbols used in an ast node.
+        """
+        if isinstance(node, ast.Name):
+            yield node.id
+
+        if isinstance(node, (ast.JoinedStr, ast.BoolOp)):
+            for n in node.values:
+                if isinstance(n, ast.Str):
+                    # NOTE: required for python3.6
+                    yield from cls.get_names_from_expression(n.s)
+                else:
+                    yield from cls.get_names_from_expression(n.value)
+
+        if isinstance(node, ast.BinOp):
+            yield from cls.get_names_from_expression(node.right)
+            yield from cls.get_names_from_expression(node.left)
+
+        if isinstance(node, ast.IfExp):
+            yield from cls.get_names_from_expression(node.test)
+            yield from cls.get_names_from_expression(node.body)
+            yield from cls.get_names_from_expression(node.orelse)
+
+        if isinstance(node, ast.Subscript):
+            yield from cls.get_names_from_expression(node.value)
+            yield from cls.get_names_from_expression(node.slice)
+
+        if isinstance(node, ast.Slice):
+            yield from cls.get_names_from_expression(node.lower)
+            yield from cls.get_names_from_expression(node.upper)
+            yield from cls.get_names_from_expression(node.step)
+
+        if isinstance(
+            node,
+            (ast.ListComp, ast.DictComp, ast.SetComp, ast.GeneratorExp)
+        ):
+            for g in node.generators:
+                yield from cls.get_names_from_expression(g.iter)
+
+        if isinstance(node, (ast.List, ast.Tuple, ast.Set)):
+            for elt in node.elts:
+                yield from cls.get_names_from_expression(elt)
+
+        if isinstance(node, ast.Dict):
+            for val in node.values:
+                yield from cls.get_names_from_expression(val)
+
+        if isinstance(node, ast.UnaryOp):
+            yield from cls.get_names_from_expression(node.operand)
+
+        if isinstance(node, ast.comprehension):
+            yield from cls.get_names_from_expression(node.iter.value)
+
+        if isinstance(node, ast.Attribute):
+            if isinstance(node.value, ast.Name):
+                yield f'{node.value.id}.{node.attr}'
+
+        if isinstance(node, ast.Call):
+            yield from cls.get_names_from_expression(node.func)
+
+            for arg in node.args:
+                yield from cls.get_names_from_expression(arg)
+            for keyword in node.keywords:
+                yield from cls.get_names_from_expression(keyword.value)
+
     def __repr__(self):
         return '<ParserRuleProperty name=%r filename=%s:%d ' \
                'value=%r watched_keys=%r>' % (
@@ -320,7 +401,7 @@ class ParserRule(object):
         for rule in re.split(lang_cls_split_pat, name):
             crule = None
 
-            if not len(rule):
+            if not rule:
                 raise ParserException(self.ctx, self.line,
                                       'Empty rule detected')
 
@@ -438,7 +519,7 @@ class Parser(object):
                     ref = ref[c:-c] if c != 2 else ref
 
                 if ref[-3:] != '.kv':
-                    Logger.warn('Lang: {0} does not have a valid Kivy'
+                    Logger.warning('Lang: {0} does not have a valid Kivy'
                                 'Language extension (.kv)'.format(ref))
                     break
                 if ref in __KV_INCLUDES__:
@@ -447,7 +528,7 @@ class Parser(object):
                                               'Invalid or unknown file: {0}'
                                               .format(ref))
                     if not force_load:
-                        Logger.warn('Lang: {0} has already been included!'
+                        Logger.warning('Lang: {0} has already been included!'
                                     .format(ref))
                         continue
                     else:
@@ -469,9 +550,10 @@ class Parser(object):
                 try:
                     if package not in sys.modules:
                         try:
-                            mod = __import__(package)
+                            mod = importlib.__import__(package)
                         except ImportError:
-                            mod = __import__('.'.join(package.split('.')[:-1]))
+                            module_name = '.'.join(package.split('.')[:-1])
+                            mod = importlib.__import__(module_name)
                         # resolve the whole thing
                         for part in package.split('.')[1:]:
                             mod = getattr(mod, part)
@@ -577,7 +659,7 @@ class Parser(object):
             # Current level, create an object
             elif count == indent:
                 x = content.split(':', 1)
-                if not len(x[0]):
+                if not x[0]:
                     raise ParserException(self, ln, 'Identifier missing')
                 if (len(x) == 2 and len(x[1]) and
                         not x[1].lstrip().startswith('#')):
@@ -597,7 +679,7 @@ class Parser(object):
             # Next level, is it a property or an object ?
             elif count == indent + spaces:
                 x = content.split(':', 1)
-                if not len(x[0]):
+                if not x[0]:
                     raise ParserException(self, ln, 'Identifier missing')
 
                 # It's a class, add to the current object as a children
@@ -613,6 +695,8 @@ class Parser(object):
                             self, ln, 'clear previous, `-`, not allowed here')
                     _objects, _lines = self.parse_level(
                         level + 1, lines[i:], spaces)
+                    if current_object is None:
+                        raise ParserException(self, ln, 'Invalid indentation')
                     current_object.children = _objects
                     lines = _lines
                     i = 0
@@ -671,6 +755,9 @@ class Parser(object):
                     if current_propobject is None:
                         current_propobject = ParserRuleProperty(
                             self, ln, current_property, content)
+                        if not current_property:
+                            raise ParserException(self, ln,
+                                                  "Invalid indentation")
                         if current_property[:3] == 'on_':
                             current_object.handlers.append(current_propobject)
                         else:
